@@ -42,7 +42,10 @@ class FaultWrapper(wsgi.Middleware):
     """Replace error body with something the client can parse."""
 
     error_map = {
+        'ActionConflict': webob.exc.HTTPConflict,
+        'ActionCooldown': webob.exc.HTTPConflict,
         'ActionInProgress': webob.exc.HTTPConflict,
+        'ActionImmutable': webob.exc.HTTPConflict,
         'BadRequest': webob.exc.HTTPBadRequest,
         'FeatureNotSupported': webob.exc.HTTPConflict,
         'Forbidden': webob.exc.HTTPForbidden,
@@ -53,8 +56,10 @@ class FaultWrapper(wsgi.Middleware):
         'MultipleChoices': webob.exc.HTTPBadRequest,
         'NodeNotOrphan': webob.exc.HTTPConflict,
         'PolicyBindingNotFound': webob.exc.HTTPNotFound,
+        'ProfileOperationFailed': webob.exc.HTTPInternalServerError,
         'RequestLimitExceeded': webob.exc.HTTPBadRequest,
         'ResourceInUse': webob.exc.HTTPConflict,
+        'ResourceIsLocked': webob.exc.HTTPConflict,
         'ResourceNotFound': webob.exc.HTTPNotFound,
     }
 
