@@ -10,15 +10,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-'''
+"""
 SDK Client
-'''
+"""
 import sys
 
 import functools
+import openstack
 from openstack import connection
 from openstack import exceptions as sdk_exc
-from openstack import utils as sdk_utils
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
@@ -32,14 +32,14 @@ USER_AGENT = 'senlin'
 exc = sdk_exc
 LOG = logging.getLogger(__name__)
 
-sdk_utils.enable_logging(debug=False, stream=sys.stdout)
+openstack.enable_logging(debug=False, stream=sys.stdout)
 
 
 def parse_exception(ex):
-    '''Parse exception code and yield useful information.'''
+    """Parse exception code and yield useful information."""
     code = 500
 
-    LOG.exception(ex)
+    LOG.error(six.text_type(ex))
     if isinstance(ex, sdk_exc.HttpException):
         # some exceptions don't contain status_code
         if hasattr(ex, "status_code") and ex.status_code is not None:
@@ -47,7 +47,7 @@ def parse_exception(ex):
         elif hasattr(ex, "http_status") and ex.http_status is not None:
             code = ex.http_status
 
-        message = ex.message
+        message = six.text_type(ex)
         data = {}
         if ex.details is None and ex.response is not None:
             data = ex.response.json()
@@ -78,7 +78,7 @@ def parse_exception(ex):
     elif isinstance(ex, sdk_exc.SDKException):
         # Besides HttpException there are some other exceptions like
         # ResourceTimeout can be raised from SDK, handle them here.
-        message = ex.message
+        message = six.text_type(ex)
     elif isinstance(ex, req_exc.RequestException):
         # Exceptions that are not captured by SDK
         code = ex.errno
@@ -98,7 +98,6 @@ def translate_exception(func):
         try:
             return func(driver, *args, **kwargs)
         except Exception as ex:
-            LOG.exception(ex)
             raise parse_exception(ex)
 
     return invoke_with_catch
@@ -125,7 +124,7 @@ def create_connection(params=None):
 
 
 def authenticate(**kwargs):
-    '''Authenticate using openstack sdk based on user credential'''
+    """Authenticate using openstack sdk based on user credential"""
 
     conn = create_connection(kwargs)
     access_info = {
@@ -138,13 +137,13 @@ def authenticate(**kwargs):
 
 
 class FakeResourceObject(object):
-    '''Generate a fake SDK resource object based on given dictionary'''
+    """Generate a fake SDK resource object based on given dictionary"""
     def __init__(self, params):
         for key in params:
             setattr(self, key, params[key])
 
     def to_dict(self):
-        '''Override this function in subclass to handle special attributes'''
+        """Override this function in subclass to handle special attributes"""
         data = {}
         for attr in dir(self):
             if not attr.startswith('__'):
