@@ -16,7 +16,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from six.moves.urllib import parse
 
-from senlin.common.i18n import _
 from senlin.engine.receivers import base
 
 LOG = logging.getLogger(__name__)
@@ -26,6 +25,7 @@ CONF = cfg.CONF
 
 class Webhook(base.Receiver):
     """Webhook flavor of receivers."""
+    WEBHOOK_VERSION = 2
 
     def initialize_channel(self, context):
         host = CONF.receiver.host
@@ -38,11 +38,11 @@ class Webhook(base.Receiver):
             base = self._get_base_url()
             if not base:
                 host = socket.gethostname()
-                msg = _('Webhook host is not specified in configuration '
-                        'file and Senlin service endpoint can not be found,'
-                        'using local hostname (%(host)s) for webhook url.'
-                        ) % {'host': host}
-                LOG.warning(msg)
+                LOG.warning(
+                    'Webhook host is not specified in configuration '
+                    'file and Senlin service endpoint can not be found,'
+                    'using local hostname (%(host)s) for webhook url.',
+                    {'host': host})
             elif base.rfind("v1") == -1:
                 base = "%s/v1" % base
 
@@ -53,9 +53,11 @@ class Webhook(base.Receiver):
         if self.params:
             normalized = sorted(self.params.items(), key=lambda d: d[0])
             qstr = parse.urlencode(normalized)
-            url = "".join([base, webhook, '?V=1&', qstr])
+            url = "".join(
+                [base, webhook, '?V={}&'.format(self.WEBHOOK_VERSION), qstr])
         else:
-            url = "".join([base, webhook, '?V=1'])
+            url = "".join(
+                [base, webhook, '?V={}'.format(self.WEBHOOK_VERSION)])
 
         self.channel = {
             'alarm_url': url
