@@ -20,7 +20,6 @@ from oslo_service import threadgroup
 from osprofiler import profiler
 
 from senlin.common import context
-from senlin.common.i18n import _
 from senlin.engine.actions import base as action_mod
 from senlin.objects import action as ao
 
@@ -30,12 +29,12 @@ wallclock = time.time
 
 
 class ThreadGroupManager(object):
-    '''Thread group manager.'''
+    """Thread group manager."""
 
     def __init__(self):
         super(ThreadGroupManager, self).__init__()
-        self.workers = {}
-        self.group = threadgroup.ThreadGroup()
+        self.group = threadgroup.ThreadGroup(
+            thread_pool_size=cfg.CONF.scheduler_thread_pool_size)
 
         # Create dummy service task, because when there is nothing queued
         # on self.tg the process exits
@@ -46,7 +45,7 @@ class ThreadGroupManager(object):
         self.db_session = context.RequestContext(is_admin=True)
 
     def _service_task(self):
-        '''Dummy task which gets queued on the service.Service threadgroup.
+        """Dummy task which gets queued on the service.Service threadgroup.
 
         Without this service.Service sees nothing running i.e has nothing to
         wait() on, so the process exits..
@@ -54,7 +53,7 @@ class ThreadGroupManager(object):
         housekeeping tasks
 
         (Yanyan)Not sure this is still necessary, just keep it temporarily.
-        '''
+        """
         # TODO(Yanyan): have this task call dbapi purge events
         pass
 
@@ -77,100 +76,96 @@ class ThreadGroupManager(object):
         return func(*args, **kwargs)
 
     def start(self, func, *args, **kwargs):
-        '''Run the given method in a thread.'''
-
+        """Run the given method in a thread."""
         req_cnxt = oslo_context.get_current()
-        return self.group.add_thread(self._start_with_trace, req_cnxt,
-                                     self._serialize_profile_info(),
-                                     func, *args, **kwargs)
+        self.group.add_thread(
+            self._start_with_trace, req_cnxt,
+            self._serialize_profile_info(),
+            func, *args, **kwargs)
 
     def start_action(self, worker_id, action_id=None):
-        '''Run action(s) in sub-thread(s).
+        """Run action(s) in sub-thread(s).
 
         :param worker_id: ID of the worker thread; we fake workers using
                           senlin engines at the moment.
         :param action_id: ID of the action to be executed. None means all
                           ready actions will be acquired and scheduled to run.
-        '''
-        def launch(action_id):
-            '''Launch a sub-thread to run given action.'''
-            th = self.start(action_mod.ActionProc, self.db_session, action_id)
-            self.workers[action_id] = th
-            th.link(release, action_id)
-            return th
-
-        def release(thread, action_id):
-            '''Callback function that will be passed to GreenThread.link().'''
-            # Remove action thread from thread list
-            self.workers.pop(action_id)
-
+        """
         actions_launched = 0
+        max_batch_size = cfg.CONF.max_actions_per_batch
+        batch_interval = cfg.CONF.batch_interval
+
         if action_id is not None:
             timestamp = wallclock()
             action = ao.Action.acquire(self.db_session, action_id, worker_id,
                                        timestamp)
             if action:
-                launch(action.id)
+                self.start(action_mod.ActionProc, self.db_session, action.id)
                 actions_launched += 1
 
-        batch_size = cfg.CONF.max_actions_per_batch
-        batch_interval = cfg.CONF.batch_interval
         while True:
             timestamp = wallclock()
             action = ao.Action.acquire_first_ready(self.db_session, worker_id,
                                                    timestamp)
-            if action:
-                if batch_size > 0 and 'NODE' in action.action:
-                    if actions_launched < batch_size:
-                        launch(action.id)
-                        actions_launched += 1
-                    else:
-                        msg = _('Engine %(id)s has launched %(num)s node '
-                                'actions consecutively, stop scheduling '
-                                'node action for %(interval)s second...'
-                                ) % {'id': worker_id, 'num': batch_size,
-                                     'interval': batch_interval}
-                        LOG.debug(msg)
-                        sleep(batch_interval)
-                        launch(action.id)
-                        actions_launched = 1
-                else:
-                    launch(action.id)
-            else:
+            if not action:
                 break
 
+            if max_batch_size == 0 or 'NODE' not in action.action:
+                self.start(action_mod.ActionProc, self.db_session, action.id)
+                continue
+
+            if max_batch_size > actions_launched:
+                self.start(action_mod.ActionProc, self.db_session, action.id)
+                actions_launched += 1
+                continue
+
+            self.start(action_mod.ActionProc, self.db_session, action.id)
+
+            LOG.debug(
+                'Engine %(id)s has launched %(num)s node actions '
+                'consecutively, stop scheduling node action for '
+                '%(interval)s second...',
+                {
+                    'id': worker_id,
+                    'num': max_batch_size,
+                    'interval': batch_interval
+                })
+
+            sleep(batch_interval)
+            actions_launched = 1
+
     def cancel_action(self, action_id):
-        '''Cancel an action execution progress.'''
+        """Cancel an action execution progress."""
         action = action_mod.Action.load(self.db_session, action_id,
                                         project_safe=False)
         action.signal(action.SIG_CANCEL)
 
     def suspend_action(self, action_id):
-        '''Suspend an action execution progress.'''
+        """Suspend an action execution progress."""
         action = action_mod.Action.load(self.db_session, action_id,
                                         project_safe=False)
         action.signal(action.SIG_SUSPEND)
 
     def resume_action(self, action_id):
-        '''Resume an action execution progress.'''
+        """Resume an action execution progress."""
         action = action_mod.Action.load(self.db_session, action_id,
                                         project_safe=False)
         action.signal(action.SIG_RESUME)
 
     def add_timer(self, interval, func, *args, **kwargs):
-        '''Define a periodic task to be run in the thread group.
+        """Define a periodic task to be run in the thread group.
 
         The task will be executed in a separate green thread.
         Interval is from cfg.CONF.periodic_interval
-        '''
-        timer = self.group.add_timer(interval, func, *args, **kwargs)
+        """
+        timer = self.group.add_timer(interval, func, None, *args, **kwargs)
         return timer
 
     def stop_timers(self):
         self.group.stop_timers()
 
     def stop(self, graceful=False):
-        '''Stop any active threads belong to this threadgroup.'''
+        """Stop any active threads belong to this threadgroup."""
         # Try to stop all threads gracefully
         self.group.stop(graceful)
         self.group.wait()
@@ -190,17 +185,17 @@ class ThreadGroupManager(object):
 
 
 def reschedule(action_id, sleep_time=1):
-    '''Eventlet Sleep for the specified number of seconds.
+    """Eventlet Sleep for the specified number of seconds.
 
     :param action_id: the action to put into sleep.
     :param sleep_time: seconds to sleep; if None, no sleep;
-    '''
+    """
     if sleep_time is not None:
-        LOG.debug('Action %s sleep for %s seconds' % (action_id, sleep_time))
+        LOG.debug('Action %s sleep for %s seconds', action_id, sleep_time)
         eventlet.sleep(sleep_time)
 
 
 def sleep(sleep_time):
-    '''Interface for sleeping.'''
+    """Interface for sleeping."""
 
     eventlet.sleep(sleep_time)
