@@ -18,7 +18,6 @@ from senlin.common import exception
 from senlin.common import utils
 from senlin.db import api as db_api
 from senlin.objects import base
-from senlin.objects import dependency as dobj
 from senlin.objects import fields
 
 
@@ -49,6 +48,8 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
         'user': fields.StringField(),
         'project': fields.StringField(),
         'domain': fields.StringField(nullable=True),
+        'dep_on': fields.CustomListField(attr_name='depended', nullable=True),
+        'dep_by': fields.CustomListField(attr_name='dependent', nullable=True),
     }
 
     @classmethod
@@ -95,6 +96,11 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
         obj = db_api.action_get_by_short_id(context, short_id, **kwargs)
         return cls._from_db_object(context, cls(), obj)
 
+    @classmethod
+    def action_list_active_scaling(cls, context, cluster_id, **kwargs):
+        objs = db_api.action_list_active_scaling(context, cluster_id, **kwargs)
+        return [cls._from_db_object(context, cls(), obj) for obj in objs]
+
     @classmethod
     def get_all(cls, context, **kwargs):
         objs = db_api.action_get_all(context, **kwargs)
@@ -105,6 +111,11 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
         objs = db_api.action_get_all_by_owner(context, owner)
         return [cls._from_db_object(context, cls(), obj) for obj in objs]
 
+    @classmethod
+    def get_all_active_by_target(cls, context, target):
+        objs = db_api.action_get_all_active_by_target(context, target)
+        return [cls._from_db_object(context, cls(), obj) for obj in objs]
+
     @classmethod
     def check_status(cls, context, action_id, timestamp):
         return db_api.action_check_status(context, action_id, timestamp)
@@ -177,12 +188,6 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
                                               status=status)
 
     def to_dict(self):
-        if self.id:
-            dep_on = dobj.Dependency.get_depended(self.context, self.id)
-            dep_by = dobj.Dependency.get_dependents(self.context, self.id)
-        else:
-            dep_on = []
-            dep_by = []
         action_dict = {
             'id': self.id,
             'name': self.name,
@@ -198,8 +203,8 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
             'status_reason': self.status_reason,
             'inputs': self.inputs,
             'outputs': self.outputs,
-            'depends_on': dep_on,
-            'depended_by': dep_by,
+            'depends_on': self.dep_on,
+            'depended_by': self.dep_by,
             'created_at': utils.isotime(self.created_at),
             'updated_at': utils.isotime(self.updated_at),
             'data': self.data,
