@@ -10,8 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import copy
-
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 from oslo_utils import strutils
@@ -79,6 +77,18 @@ class NonNegativeInteger(fields.FieldType):
         }
 
 
+# Senlin has a stricter field checking for object fields.
+class Object(fields.Object):
+
+    def get_schema(self):
+        schema = super(Object, self).get_schema()
+        # we are not checking whether self._obj_name is registered, an
+        # exception will be raised anyway if it is not registered.
+        data_key = 'senlin_object.data'
+        schema['properties'][data_key]['additionalProperties'] = False
+        return schema
+
+
 class UUID(fields.FieldType):
 
     _PATTERN = (r'^[a-fA-F0-9]{8}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]'
@@ -97,60 +107,6 @@ class UUID(fields.FieldType):
         return {'type': ['string'], 'pattern': self._PATTERN}
 
 
-# TODO(Qiming): remove this when oslo patch is released
-# i.e. global requirements bump to 1.17.0
-class Object(fields.Object):
-
-    def get_schema(self):
-        from oslo_versionedobjects import base as obj_base
-        obj_classes = obj_base.VersionedObjectRegistry.obj_classes()
-        if self._obj_name in obj_classes:
-            cls = obj_classes[self._obj_name][0]
-            namespace_key = cls._obj_primitive_key('namespace')
-            name_key = cls._obj_primitive_key('name')
-            version_key = cls._obj_primitive_key('version')
-            data_key = cls._obj_primitive_key('data')
-            changes_key = cls._obj_primitive_key('changes')
-            field_schemas = {key: field.get_schema()
-                             for key, field in cls.fields.items()}
-            required_fields = [key for key, field in sorted(cls.fields.items())
-                               if not field.nullable]
-
-            schema = {
-                'type': 'object',
-                'properties': {
-                    namespace_key: {
-                        'type': 'string',
-                    },
-                    name_key: {
-                        'type': 'string',
-                    },
-                    version_key: {
-                        'type': 'string',
-                    },
-                    changes_key: {
-                        'type': 'array',
-                        'items': {
-                            'type': 'string',
-                        }
-                    },
-                    data_key: {
-                        'type': 'object',
-                        'description': 'fields of %s' % self._obj_name,
-                        'properties': field_schemas,
-                        'additionalProperties': False,
-                    },
-                },
-                'required': [namespace_key, name_key, version_key, data_key],
-            }
-            if required_fields:
-                schema['properties'][data_key]['required'] = required_fields
-
-            return schema
-        else:
-            return {}
-
-
 class Json(fields.FieldType):
     def coerce(self, obj, attr, value):
         if isinstance(value, six.string_types):
@@ -216,10 +172,13 @@ class Name(fields.String):
         else:
             # NOTE: This is pretty restrictive. We can relax it later when
             # there are requests to do so
-            regex = re.compile('^[a-zA-Z\d\.\_\~-]*$', re.IGNORECASE)
+            regex = re.compile(u'^[a-zA-Z\u4e00-\u9fa5\d\.\_\~-]*$',
+                               re.IGNORECASE)
             if not regex.search(value):
                 err = _("The value for the '%(attr)s' (%(value)s) contains "
-                        "illegal characters."
+                        "illegal characters. It must contain only "
+                        "alphanumeric or \"_-.~\" characters and must start "
+                        "with letter."
                         ) % {'attr': attr, 'value': value}
 
         if err:
@@ -380,34 +339,24 @@ class BaseEnum(fields.FieldType):
             return None
         return '\'%s\'' % value
 
-    def get_schema(self):
-        # TODO(Anyone): remove this override when the following patch is
-        # merged and released.
-        # https://review.openstack.org/389061
-        valid_values = copy.deepcopy(self._valid_values)
-        if self._nullable:
-            valid_values.append(None)
-
-        schema = {
-            'type': ['string'],
-            'enum': valid_values,
-        }
-        return schema
-
 
 class AdjustmentType(BaseEnum):
 
-    ALL = (
-        EXACT_CAPACITY, CHANGE_IN_CAPACITY, CHANGE_IN_PERCENTAGE,
-    ) = (
-        'EXACT_CAPACITY', 'CHANGE_IN_CAPACITY', 'CHANGE_IN_PERCENTAGE',
-    )
+    ALL = consts.ADJUSTMENT_TYPES
+
+    def get_schema(self):
+        return {'type': ['string'],
+                'enum': self._valid_values}
 
 
 class ClusterActionName(BaseEnum):
 
     ALL = consts.CLUSTER_ACTION_NAMES
 
+    def get_schema(self):
+        return {'type': ['string'],
+                'enum': self._valid_values}
+
 
 class ClusterStatus(BaseEnum):
 
@@ -428,6 +377,10 @@ class ReceiverType(BaseEnum):
 
     ALL = consts.RECEIVER_TYPES
 
+    def get_schema(self):
+        return {'type': ['string'],
+                'enum': self._valid_values}
+
 
 class UniqueDict(fields.Dict):
 
@@ -451,6 +404,8 @@ class BooleanField(fields.AutoTypedField):
     AUTO_TYPE = Boolean()
 
 
+# An override to the oslo.versionedobjects version so that we are using
+# our own Object definition.
 class ObjectField(fields.AutoTypedField):
 
     def __init__(self, objtype, subclasses=False, **kwargs):
@@ -564,3 +519,17 @@ class ReceiverTypeField(fields.AutoTypedField):
 class NodeReplaceMapField(fields.AutoTypedField):
 
     AUTO_TYPE = UniqueDict(fields.String())
+
+
+class CustomListField(ListField):
+
+    def __init__(self, attr_name, **kwargs):
+        self.attr_name = attr_name
+        super(CustomListField, self).__init__(**kwargs)
+
+    def coerce(self, obj, attr, value):
+        objs = super(CustomListField, self).coerce(obj, attr, value)
+        custom_list = []
+        for i in objs:
+            custom_list.append(getattr(i, self.attr_name))
+        return custom_list
