@@ -11,8 +11,10 @@
 # under the License.
 
 import copy
+import eventlet
 import inspect
 
+from oslo_config import cfg
 from oslo_context import context as oslo_context
 from oslo_log import log as logging
 from oslo_utils import timeutils
@@ -127,13 +129,14 @@ class Profile(object):
         self._orchestrationclient = None
         self._workflowclient = None
         self._block_storageclient = None
+        self._glanceclient = None
 
     @classmethod
     def _from_object(cls, profile):
-        '''Construct a profile from profile object.
+        """Construct a profile from profile object.
 
         :param profile: a profile object that contains all required fields.
-        '''
+        """
         kwargs = {
             'id': profile.id,
             'type': profile.type,
@@ -150,7 +153,7 @@ class Profile(object):
 
     @classmethod
     def load(cls, ctx, profile=None, profile_id=None, project_safe=True):
-        '''Retrieve a profile object from database.'''
+        """Retrieve a profile object from database."""
         if profile is None:
             profile = po.Profile.get(ctx, profile_id,
                                      project_safe=project_safe)
@@ -191,7 +194,7 @@ class Profile(object):
         po.Profile.delete(ctx, profile_id)
 
     def store(self, ctx):
-        '''Store the profile into database and return its ID.'''
+        """Store the profile into database and return its ID."""
         timestamp = timeutils.utcnow(True)
 
         values = {
@@ -299,11 +302,13 @@ class Profile(object):
     @profiler.trace('Profile.check_object', hide_args=False)
     def check_object(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
-        try:
-            return profile.do_check(obj)
-        except exc.InternalError as ex:
-            LOG.error(ex)
-            return False
+        return profile.do_check(obj)
+
+    @classmethod
+    @profiler.trace('Profile.check_object', hide_args=False)
+    def healthcheck_object(cls, ctx, obj):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        return profile.do_healthcheck(obj)
 
     @classmethod
     @profiler.trace('Profile.recover_object', hide_args=False)
@@ -372,12 +377,12 @@ class Profile(object):
         return params
 
     def compute(self, obj):
-        '''Construct compute client based on object.
+        """Construct compute client based on object.
 
         :param obj: Object for which the client is created. It is expected to
                     be None when retrieving an existing client. When creating
                     a client, it contains the user and project to be used.
-        '''
+        """
 
         if self._computeclient is not None:
             return self._computeclient
@@ -385,6 +390,19 @@ class Profile(object):
         self._computeclient = driver_base.SenlinDriver().compute(params)
         return self._computeclient
 
+    def glance(self, obj):
+        """Construct glance client based on object.
+
+        :param obj: Object for which the client is created. It is expected to
+                    be None when retrieving an existing client. When creating
+                    a client, it contains the user and project to be used.
+        """
+        if self._glanceclient is not None:
+            return self._glanceclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._glanceclient = driver_base.SenlinDriver().glance(params)
+        return self._glanceclient
+
     def network(self, obj):
         """Construct network client based on object.
 
@@ -459,6 +477,18 @@ class Profile(object):
         LOG.warning("Check operation not supported.")
         return True
 
+    def do_healthcheck(self, obj):
+        """Default healthcheck operation.
+
+        This is provided as a fallback if a specific profile type does not
+        override this method.
+
+        :param obj: The node object to operate on.
+        :return status: True indicates node is healthy, False indicates
+            it is unhealthy.
+        """
+        return self.do_check(obj)
+
     def do_get_details(self, obj):
         """For subclass to override."""
         LOG.warning("Get_details operation not supported.")
@@ -487,31 +517,50 @@ class Profile(object):
 
         :param obj: The node object to operate on.
         :param options: Keyword arguments for the recover operation.
+        :return id: New id of the recovered resource or None if recovery
+            failed.
+        :return status: True indicates successful recovery, False indicates
+            failure.
         """
-        operation = options.pop('operation', None)
+        operation = options.get('operation', None)
+        force_recreate = options.get('force_recreate', None)
+        delete_timeout = options.get('delete_timeout', None)
 
-        # The operation is a list of action names with optional parameters
-        if operation and not isinstance(operation, six.string_types):
-            operation = operation[0]
-
-        if operation and operation['name'] != consts.RECOVER_RECREATE:
+        if operation.upper() != consts.RECOVER_RECREATE:
             LOG.error("Recover operation not supported: %s", operation)
-            return False
+            return None, False
+
+        extra_params = options.get('operation_params', None)
+        fence_compute = False
+        if extra_params:
+            fence_compute = extra_params.get('fence_compute', False)
 
-        extra_params = options.get('params', {})
-        fence_compute = extra_params.get('fence_compute', False)
         try:
-            self.do_delete(obj, force=fence_compute)
+            self.do_delete(obj, force=fence_compute, timeout=delete_timeout)
         except exc.EResourceDeletion as ex:
-            raise exc.EResourceOperation(op='recovering', type='node',
-                                         id=obj.id, message=six.text_type(ex))
+            if force_recreate:
+                # log error and continue on to creating the node
+                LOG.warning('Failed to delete node during recovery action: %s',
+                            ex)
+            else:
+                raise exc.EResourceOperation(op='recovering', type='node',
+                                             id=obj.id,
+                                             message=six.text_type(ex))
+
+        # pause to allow deleted resource to get reclaimed by nova
+        # this is needed to avoid a problem when the compute resources are
+        # at their quota limit.  The deleted resource has to become available
+        # so that the new node can be created.
+        eventlet.sleep(cfg.CONF.batch_interval)
+
         res = None
         try:
             res = self.do_create(obj)
         except exc.EResourceCreation as ex:
             raise exc.EResourceOperation(op='recovering', type='node',
-                                         id=obj.id, message=six.text_type(ex))
-        return res
+                                         id=obj.id, message=six.text_type(ex),
+                                         resource_id=ex.resource_id)
+        return res, True
 
     def do_validate(self, obj):
         """For subclass to override."""
