@@ -37,7 +37,7 @@ class NodeActionTest(base.SenlinTestCase):
 
     def test_do_create_okay(self, mock_load):
         node = mock.Mock(id='NID')
-        node.do_create = mock.Mock(return_value=True)
+        node.do_create = mock.Mock(return_value=[True, ''])
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx)
 
@@ -49,7 +49,8 @@ class NodeActionTest(base.SenlinTestCase):
 
     def test_do_create_failed(self, mock_load):
         node = mock.Mock(id='NID')
-        node.do_create = mock.Mock(return_value=False)
+        node.do_create = mock.Mock(return_value=[False,
+                                                 'custom error message'])
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx)
 
@@ -57,7 +58,7 @@ class NodeActionTest(base.SenlinTestCase):
         res_code, res_msg = action.do_create()
 
         self.assertEqual(action.RES_ERROR, res_code)
-        self.assertEqual('Node creation failed.', res_msg)
+        self.assertEqual('custom error message', res_msg)
         node.do_create.assert_called_once_with(action.context)
 
     @mock.patch.object(scaleutils, 'check_size_params')
@@ -68,7 +69,7 @@ class NodeActionTest(base.SenlinTestCase):
         cluster = mock.Mock(id='CID')
         mock_c_load.return_value = cluster
         node = mock.Mock(id='NID', cluster_id='CID')
-        node.do_create = mock.Mock(return_value=True)
+        node.do_create = mock.Mock(return_value=[True, ''])
         mock_load.return_value = node
         mock_count.return_value = 11
         mock_check.return_value = None
@@ -97,7 +98,7 @@ class NodeActionTest(base.SenlinTestCase):
         cluster = mock.Mock(id='CID')
         mock_c_load.return_value = cluster
         node = mock.Mock(id='NID', cluster_id='CID')
-        node.do_create = mock.Mock(return_value=True)
+        node.do_create = mock.Mock(return_value=[True, ''])
         mock_load.return_value = node
         mock_count.return_value = 11
         mock_check.return_value = 'overflow'
@@ -114,7 +115,8 @@ class NodeActionTest(base.SenlinTestCase):
         mock_count.assert_called_once_with(action.context, 'CID')
         mock_check.assert_called_once_with(cluster, 11, None, None, True)
         mock_update.assert_called_once_with(action.context, 'NID',
-                                            {'cluster_id': ''})
+                                            {'cluster_id': '',
+                                             'status': consts.NS_ERROR})
         self.assertEqual(0, node.do_create.call_count)
         self.assertEqual(0, cluster.eval_status.call_count)
 
@@ -127,7 +129,8 @@ class NodeActionTest(base.SenlinTestCase):
         cluster = mock.Mock(id='CID')
         mock_c_load.return_value = cluster
         node = mock.Mock(id='NID', cluster_id='CID')
-        node.do_create = mock.Mock(return_value=False)
+        node.do_create = mock.Mock(return_value=[False,
+                                                 'custom error message'])
         mock_load.return_value = node
         mock_count.return_value = 11
         mock_check.return_value = ''
@@ -139,7 +142,7 @@ class NodeActionTest(base.SenlinTestCase):
 
         # assertions
         self.assertEqual(action.RES_ERROR, res_code)
-        self.assertEqual('Node creation failed.', res_msg)
+        self.assertEqual('custom error message', res_msg)
         mock_c_load.assert_called_once_with(action.context, 'CID')
         mock_count.assert_called_once_with(action.context, 'CID')
         mock_check.assert_called_once_with(cluster, 11, None, None, True)
@@ -407,6 +410,22 @@ class NodeActionTest(base.SenlinTestCase):
         self.assertEqual('Node updated successfully.', res_msg)
         node.do_update.assert_called_once_with(action.context, inputs)
 
+    def test_do_update_no_need_update(self, mock_load):
+        node = mock.Mock()
+        node.id = 'NID'
+        node.profile_id = 'PROFILE_ID'
+        mock_load.return_value = node
+        inputs = {"new_profile_id": "PROFILE_ID"}
+        action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
+                                        inputs=inputs)
+
+        # Test node update success path
+        node.do_update = mock.Mock(return_value=mock.Mock())
+        res_code, res_msg = action.do_update()
+        self.assertEqual(action.RES_OK, res_code)
+        self.assertEqual('No property to update.', res_msg)
+        self.assertFalse(node.do_update.called)
+
     def test_do_join_success(self, mock_load):
         node = mock.Mock(id='NID')
         mock_load.return_value = node
@@ -697,6 +716,52 @@ class NodeActionTest(base.SenlinTestCase):
                                                   'ACTION_ID', None, False)
         mock_release_node.assert_called_once_with('NODE_ID', 'ACTION_ID')
 
+    @mock.patch.object(lock, 'cluster_lock_acquire')
+    @mock.patch.object(lock, 'cluster_lock_release')
+    @mock.patch.object(base_action.Action, 'policy_check')
+    @mock.patch.object(lock, 'node_lock_acquire')
+    @mock.patch.object(lock, 'node_lock_release')
+    def test_execute_success_stealing_node_lock(self, mock_release_node,
+                                                mock_acquire_node, mock_check,
+                                                mock_release, mock_acquire,
+                                                mock_load):
+        node = mock.Mock()
+        node.cluster_id = 'FAKE_CLUSTER'
+        node.id = 'NODE_ID'
+        mock_load.return_value = node
+
+        action = node_action.NodeAction('NODE_ID', 'NODE_OPERATION', self.ctx,
+                                        cause='RPC Request')
+        action.id = 'ACTION_ID'
+        action.data = {
+            'status': policy_mod.CHECK_OK,
+            'reason': 'Policy checking passed'
+        }
+        action.inputs = {'operation': 'stop', 'params': {}}
+
+        mock_acquire.return_value = 'ACTION_ID'
+        mock_acquire_node.return_value = True
+
+        res_code, res_msg = action.execute()
+
+        reason = "Node operation 'stop' succeeded."
+        self.assertEqual(action.RES_OK, res_code)
+        self.assertEqual(reason, res_msg)
+        mock_load.assert_called_once_with(action.context, node_id='NODE_ID')
+        mock_acquire.assert_called_once_with(self.ctx, 'FAKE_CLUSTER',
+                                             'ACTION_ID', None,
+                                             lock.NODE_SCOPE, False)
+        policy_calls = [
+            mock.call('FAKE_CLUSTER', 'BEFORE'),
+            mock.call('FAKE_CLUSTER', 'AFTER')
+        ]
+        mock_release.assert_called_once_with('FAKE_CLUSTER', 'ACTION_ID',
+                                             lock.NODE_SCOPE)
+        mock_check.assert_has_calls(policy_calls)
+        mock_acquire_node.assert_called_once_with(self.ctx, 'NODE_ID',
+                                                  'ACTION_ID', None, True)
+        mock_release_node.assert_called_once_with('NODE_ID', 'ACTION_ID')
+
     @mock.patch.object(lock, 'cluster_lock_acquire')
     @mock.patch.object(lock, 'cluster_lock_release')
     @mock.patch.object(base_action.Action, 'policy_check')
@@ -786,7 +851,11 @@ class NodeActionTest(base.SenlinTestCase):
         mock_acquire_node.assert_called_once_with(self.ctx, 'NODE_ID',
                                                   'ACTION_ID', None, False)
         mock_release_node.assert_called_once_with('NODE_ID', 'ACTION_ID')
-        mock_check.assert_called_once_with('FAKE_CLUSTER', 'BEFORE')
+        check_calls = [
+            mock.call('FAKE_CLUSTER', 'BEFORE'),
+            mock.call('FAKE_CLUSTER', 'AFTER')
+        ]
+        mock_check.assert_has_calls(check_calls)
 
     @mock.patch.object(lock, 'cluster_lock_acquire')
     @mock.patch.object(lock, 'cluster_lock_release')
